@@ -23,13 +23,15 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from PyQt5.QtGui import *
-from PyQt5.QtCore import *
-from PyQt5.QtWidgets import *
 from electrum.i18n import _
+from electrum.mnemonic import Mnemonic
+import electrum.old_mnemonic
+from electrum.plugins import run_hook
+
 
 from .util import *
 from .qrtextedit import ShowQRTextEdit, ScanQRTextEdit
+from .completion_text_edit import CompletionTextEdit
 
 
 def seed_warning_msg(seed):
@@ -49,9 +51,6 @@ def seed_warning_msg(seed):
 
 
 class SeedLayout(QVBoxLayout):
-    #options
-    is_bip39 = False
-    is_ext = False
 
     def seed_options(self):
         dialog = QDialog()
@@ -86,26 +85,34 @@ class SeedLayout(QVBoxLayout):
         self.is_ext = cb_ext.isChecked() if 'ext' in self.options else False
         self.is_bip39 = cb_bip39.isChecked() if 'bip39' in self.options else False
 
-    def __init__(self, seed=None, title=None, icon=True, msg=None, options=None, is_seed=None, passphrase=None, parent=None):
+    def __init__(self, seed=None, title=None, icon=True, msg=None, options=None,
+                 is_seed=None, passphrase=None, parent=None, for_seed_words=True):
         QVBoxLayout.__init__(self)
         self.parent = parent
         self.options = options
         if title:
             self.addWidget(WWLabel(title))
-        if seed:
-            self.seed_e = ShowQRTextEdit()
+        if seed:  # "read only", we already have the text
+            if for_seed_words:
+                self.seed_e = ButtonsTextEdit()
+            else:  # e.g. xpub
+                self.seed_e = ShowQRTextEdit()
+            self.seed_e.setReadOnly(True)
             self.seed_e.setText(seed)
-        else:
-            self.seed_e = ScanQRTextEdit()
-            self.seed_e.setTabChangesFocus(True)
+        else:  # we expect user to enter text
+            assert for_seed_words
+            self.seed_e = CompletionTextEdit()
+            self.seed_e.setTabChangesFocus(False)  # so that tab auto-completes
             self.is_seed = is_seed
             self.saved_is_seed = self.is_seed
             self.seed_e.textChanged.connect(self.on_edit)
+            self.initialize_completer()
+
         self.seed_e.setMaximumHeight(75)
         hbox = QHBoxLayout()
         if icon:
             logo = QLabel()
-            logo.setPixmap(QPixmap(":icons/seed.png").scaledToWidth(64))
+            logo.setPixmap(QPixmap(":icons/seed.png").scaledToWidth(64, mode=Qt.SmoothTransformation))
             logo.setMaximumWidth(60)
             hbox.addWidget(logo)
         hbox.addWidget(self.seed_e)
@@ -114,6 +121,10 @@ class SeedLayout(QVBoxLayout):
         hbox.addStretch(1)
         self.seed_type_label = QLabel('')
         hbox.addWidget(self.seed_type_label)
+
+        # options
+        self.is_bip39 = False
+        self.is_ext = False
         if options:
             opt_button = EnterButton(_('Options'), self.seed_options)
             hbox.addWidget(opt_button)
@@ -132,6 +143,14 @@ class SeedLayout(QVBoxLayout):
             self.seed_warning.setText(seed_warning_msg(seed))
         self.addWidget(self.seed_warning)
 
+    def initialize_completer(self):
+        english_list = Mnemonic('en').wordlist
+        old_list = electrum.old_mnemonic.words
+        self.wordlist = english_list + list(set(old_list) - set(english_list)) #concat both lists
+        self.wordlist.sort()
+        self.completer = QCompleter(self.wordlist)
+        self.seed_e.set_completer(self.completer)
+
     def get_seed(self):
         text = self.seed_e.text()
         return ' '.join(text.split())
@@ -151,15 +170,24 @@ class SeedLayout(QVBoxLayout):
         self.seed_type_label.setText(label)
         self.parent.next_button.setEnabled(b)
 
+        # to account for bip39 seeds
+        for word in self.get_seed().split(" ")[:-1]:
+            if word not in self.wordlist:
+                self.seed_e.disable_suggestions()
+                return
+        self.seed_e.enable_suggestions()
 
 class KeysLayout(QVBoxLayout):
-    def __init__(self, parent=None, title=None, is_valid=None):
+    def __init__(self, parent=None, header_layout=None, is_valid=None, allow_multi=False):
         QVBoxLayout.__init__(self)
         self.parent = parent
         self.is_valid = is_valid
-        self.text_e = ScanQRTextEdit()
+        self.text_e = ScanQRTextEdit(allow_multi=allow_multi)
         self.text_e.textChanged.connect(self.on_edit)
-        self.addWidget(WWLabel(title))
+        if isinstance(header_layout, str):
+            self.addWidget(WWLabel(header_layout))
+        else:
+            self.addLayout(header_layout)
         self.addWidget(self.text_e)
 
     def get_text(self):
@@ -179,4 +207,5 @@ class SeedDialog(WindowModalDialog):
         title =  _("Your wallet generation seed is:")
         slayout = SeedLayout(title=title, seed=seed, msg=True, passphrase=passphrase)
         vbox.addLayout(slayout)
+        run_hook('set_seed', seed, slayout.seed_e)
         vbox.addLayout(Buttons(CloseButton(self)))
