@@ -5,8 +5,12 @@
 
 try:
     import electrum
-    from electrum.bitcoin import TYPE_ADDRESS, push_script, var_int, msg_magic, Hash, verify_message, pubkey_from_signature, point_to_ser, public_key_to_p2pkh, EncodeAES, DecodeAES, MyVerifyingKey
-    from electrum.bitcoin import serialize_xpub, deserialize_xpub
+    from electrum.crypto import Hash, EncodeAES, DecodeAES
+    from electrum.bitcoin import (TYPE_ADDRESS, push_script, var_int, public_key_to_p2pkh, is_address,
+                                  serialize_xpub, deserialize_xpub)
+    from electrum import ecc
+    from electrum.ecc import msg_magic
+    from electrum.wallet import Standard_Wallet
     from electrum import constants
     from electrum.transaction import Transaction
     from electrum.i18n import _
@@ -26,9 +30,6 @@ try:
     import base64
     import os
     import sys
-    from ecdsa.ecdsa import generator_secp256k1
-    from ecdsa.util import sigencode_der
-    from ecdsa.curves import SECP256k1
     DIGIBOX = True
 except ImportError as e:
     DIGIBOX = False
@@ -82,13 +83,20 @@ class DigitalBitbox_Client():
     def is_paired(self):
         return self.password is not None
 
+    def has_usable_connection_with_device(self):
+        try:
+            self.dbb_has_password()
+        except BaseException:
+            return False
+        return True
+
     def _get_xpub(self, bip32_path):
         if self.check_device_dialog():
-            return self.hid_send_encrypt(b'{"xpub": "%s"}' % bip32_path.encode('utf8'))
+            return self.hid_send_encrypt(('{"xpub": "%s"}' % bip32_path).encode('utf8'))
 
 
     def get_xpub(self, bip32_path, xtype):
-        assert xtype in ('standard', 'p2wpkh-p2sh')
+        assert xtype in self.plugin.SUPPORTED_XTYPES
         reply = self._get_xpub(bip32_path)
         if reply:
             xpub = reply['xpub']
@@ -100,7 +108,7 @@ class DigitalBitbox_Client():
                 xpub = serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
             return xpub
         else:
-            raise BaseException('no reply')
+            raise Exception('no reply')
 
 
     def dbb_has_password(self):
@@ -114,7 +122,7 @@ class DigitalBitbox_Client():
 
     def stretch_key(self, key):
         import pbkdf2, hmac
-        return binascii.hexlify(pbkdf2.PBKDF2(key, b'Digital Bitbox', iterations = 20480, macmodule = hmac, digestmodule = hashlib.sha512).read(64))
+        return to_hexstr(pbkdf2.PBKDF2(key, b'Digital Bitbox', iterations = 20480, macmodule = hmac, digestmodule = hashlib.sha512).read(64))
 
 
     def backup_password_dialog(self):
@@ -168,7 +176,7 @@ class DigitalBitbox_Client():
         msg = _("Enter your Digital Bitbox password:")
         while self.password is None:
             if not self.password_dialog(msg):
-                return False
+                raise UserCancelled()
             reply = self.hid_send_encrypt(b'{"led":"blink"}')
             if 'error' in reply:
                 self.password = None
@@ -247,10 +255,15 @@ class DigitalBitbox_Client():
         if not dbb_user_dir:
             return
 
+        try:
+            # Python 3.5+
+            jsonDecodeError = json.JSONDecodeError
+        except AttributeError:
+            jsonDecodeError = ValueError
         try:
             with open(os.path.join(dbb_user_dir, "config.dat")) as f:
                 dbb_config = json.load(f)
-        except (FileNotFoundError, json.JSONDecodeError):
+        except (FileNotFoundError, jsonDecodeError):
             return
 
         if 'encryptionprivkey' not in dbb_config or 'comserverchannelid' not in dbb_config:
@@ -258,7 +271,7 @@ class DigitalBitbox_Client():
 
         choices = [
             _('Do not pair'),
-            _('Import pairing from the digital bitbox desktop app'),
+            _('Import pairing from the Digital Bitbox desktop app'),
         ]
         try:
             reply = self.handler.win.query_choice(_('Mobile pairing options'), choices)
@@ -277,8 +290,8 @@ class DigitalBitbox_Client():
 
     def dbb_generate_wallet(self):
         key = self.stretch_key(self.password)
-        filename = ("Electrum-" + time.strftime("%Y-%m-%d-%H-%M-%S") + ".pdf").encode('utf8')
-        msg = b'{"seed":{"source": "create", "key": "%s", "filename": "%s", "entropy": "%s"}}' % (key, filename, b'Digital Bitbox Electrum Plugin')
+        filename = ("Electrum-" + time.strftime("%Y-%m-%d-%H-%M-%S") + ".pdf")
+        msg = ('{"seed":{"source": "create", "key": "%s", "filename": "%s", "entropy": "%s"}}' % (key, filename, 'Digital Bitbox Electrum Plugin')).encode('utf8')
         reply = self.hid_send_encrypt(msg)
         if 'error' in reply:
             raise Exception(reply['error']['message'])
@@ -313,7 +326,7 @@ class DigitalBitbox_Client():
             self.handler.show_message(_("Loading backup...") + "\n\n" +
                                       _("To continue, touch the Digital Bitbox's light for 3 seconds.") + "\n\n" +
                                       _("To cancel, briefly touch the light or wait for the timeout."))
-        msg = b'{"seed":{"source": "backup", "key": "%s", "filename": "%s"}}' % (key, backups['backup'][f].encode('utf8'))
+        msg = ('{"seed":{"source": "backup", "key": "%s", "filename": "%s"}}' % (key, backups['backup'][f])).encode('utf8')
         hid_reply = self.hid_send_encrypt(msg)
         self.handler.finished()
         if 'error' in hid_reply:
@@ -441,7 +454,7 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
             hasharray.append({'hash': inputHash, 'keypath': inputPath})
             hasharray = json.dumps(hasharray)
 
-            msg = b'{"sign":{"meta":"sign message", "data":%s}}' % hasharray.encode('utf8')
+            msg = ('{"sign":{"meta":"sign message", "data":%s}}' % hasharray).encode('utf8')
 
             dbb_client = self.plugin.get_client(self)
 
@@ -463,19 +476,21 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
 
             if 'recid' in reply['sign'][0]:
                 # firmware > v2.1.1
-                sig = bytes([27 + int(reply['sign'][0]['recid'], 16) + 4]) + binascii.unhexlify(reply['sign'][0]['sig'])
-                pk, compressed = pubkey_from_signature(sig, msg_hash)
-                pk = point_to_ser(pk.pubkey.point, compressed)
-                addr = public_key_to_p2pkh(pk)
-                if verify_message(addr, sig, message) is False:
+                sig_string = binascii.unhexlify(reply['sign'][0]['sig'])
+                recid = int(reply['sign'][0]['recid'], 16)
+                sig = ecc.construct_sig65(sig_string, recid, True)
+                pubkey, compressed = ecc.ECPubkey.from_signature65(sig, msg_hash)
+                addr = public_key_to_p2pkh(pubkey.get_public_key_bytes(compressed=compressed))
+                if ecc.verify_message_with_address(addr, sig, message) is False:
                     raise Exception(_("Could not sign message"))
             elif 'pubkey' in reply['sign'][0]:
                 # firmware <= v2.1.1
-                for i in range(4):
-                    sig = bytes([27 + i + 4]) + binascii.unhexlify(reply['sign'][0]['sig'])
+                for recid in range(4):
+                    sig_string = binascii.unhexlify(reply['sign'][0]['sig'])
+                    sig = ecc.construct_sig65(sig_string, recid, True)
                     try:
                         addr = public_key_to_p2pkh(binascii.unhexlify(reply['sign'][0]['pubkey']))
-                        if verify_message(addr, sig, message):
+                        if ecc.verify_message_with_address(addr, sig, message):
                             break
                     except Exception:
                         continue
@@ -544,7 +559,7 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
                             # expected serialization though, so we leave it here until we activate it.
                             return '00' + push_script(Transaction.get_preimage_script(txin))
                         raise Exception("unsupported type %s" % txin['type'])
-                tx_dbb_serialized = CustomTXSerialization(tx.serialize()).serialize()
+                tx_dbb_serialized = CustomTXSerialization(tx.serialize()).serialize_to_network()
             else:
                 # We only need this for the signing echo / verification.
                 tx_dbb_serialized = None
@@ -621,8 +636,8 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
                         recid = int(signed['recid'], 16)
                         s = binascii.unhexlify(signed['sig'])
                         h = inputhasharray[i]
-                        pk = MyVerifyingKey.from_signature(s, recid, h, curve = SECP256k1)
-                        pk = to_hexstr(point_to_ser(pk.pubkey.point, True))
+                        pk = ecc.ECPubkey.from_sig_string(s, recid, h)
+                        pk = pk.get_public_key_hex(compressed=True)
                     elif 'pubkey' in signed:
                         # firmware <= v2.1.1
                         pk = signed['pubkey']
@@ -630,9 +645,9 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
                         continue
                     sig_r = int(signed['sig'][:64], 16)
                     sig_s = int(signed['sig'][64:], 16)
-                    sig = sigencode_der(sig_r, sig_s, generator_secp256k1.order())
-                    txin['signatures'][ii] = to_hexstr(sig) + '01'
-                    tx._inputs[i] = txin
+                    sig = ecc.der_sig_from_r_and_s(sig_r, sig_s)
+                    sig = to_hexstr(sig) + '01'
+                    tx.add_signature_to_txin(i, ii, sig)
         except UserCancelled:
             raise
         except BaseException as e:
@@ -650,6 +665,7 @@ class DigitalBitboxPlugin(HW_PluginBase):
     DEVICE_IDS = [
                    (0x03eb, 0x2402) # Digital Bitbox
                  ]
+    SUPPORTED_XTYPES = ('standard', 'p2wpkh-p2sh', 'p2wpkh', 'p2wsh-p2sh', 'p2wsh')
 
     def __init__(self, parent, config, name):
         HW_PluginBase.__init__(self, parent, config, name)
@@ -681,6 +697,9 @@ class DigitalBitboxPlugin(HW_PluginBase):
         devmgr = self.device_manager()
         device_id = device_info.device.id_
         client = devmgr.client_by_id(device_id)
+        if client is None:
+            raise Exception(_('Failed to create a client for this device.') + '\n' +
+                            _('Make sure it is in the correct state.'))
         client.handler = self.create_handler(wizard)
         if purpose == HWD_SETUP_NEW_WALLET:
             client.setupRunning = True
@@ -706,8 +725,8 @@ class DigitalBitboxPlugin(HW_PluginBase):
 
 
     def get_xpub(self, device_id, derivation, xtype, wizard):
-        if xtype not in ('standard', 'p2wpkh-p2sh'):
-            raise ScriptTypeNotSupported(_('This type of script is not supported with the Digital Bitbox.'))
+        if xtype not in self.SUPPORTED_XTYPES:
+            raise ScriptTypeNotSupported(_('This type of script is not supported with {}.').format(self.device))
         devmgr = self.device_manager()
         client = devmgr.client_by_id(device_id)
         client.handler = self.create_handler(wizard)
@@ -724,3 +743,26 @@ class DigitalBitboxPlugin(HW_PluginBase):
         if client is not None:
             client.check_device_dialog()
         return client
+
+    def show_address(self, wallet, address, keystore=None):
+        if keystore is None:
+            keystore = wallet.get_keystore()
+        if not self.show_address_helper(wallet, address, keystore):
+            return
+        if type(wallet) is not Standard_Wallet:
+            keystore.handler.show_error(_('This function is only available for standard wallets when using {}.').format(self.device))
+            return
+        if not self.is_mobile_paired():
+            keystore.handler.show_error(_('This function is only available after pairing your {} with a mobile device.').format(self.device))
+            return
+        if not keystore.is_p2pkh():
+            keystore.handler.show_error(_('This function is only available for p2pkh keystores when using {}.').format(self.device))
+            return
+        change, index = wallet.get_address_index(address)
+        keypath = '%s/%d/%d' % (keystore.derivation, change, index)
+        xpub = self.get_client(keystore)._get_xpub(keypath)
+        verify_request_payload = {
+            "type": 'p2pkh',
+            "echo": xpub['echo'],
+        }
+        self.comserver_post_notification(verify_request_payload)
